%-------------------------------------------------------------------------------------------------
% Triangle Counting
% Author : Sai Akhil P
% Input  : Edge List of a directed graph with nodes labelled by numbers
% Output : Number of Triangles in the graph
% This function, tricountdir calculates the number of directed triangles in a directed graph
% Assuming no Parallel Edges in the graph
%-------------------------------------------------------------------------------------------------

function number_of_triangles = tricountdir(edgeList)

% Validating Input
sizeofA = size(edgeList);
if sizeofA(2)~=2 && sizeofA(1) ~= 2 % Check if the input has either two rows or two columns
    error('Input must be an edge list')
end
if ~isnumeric(edgeList) % Check if input is numeric
    error('Nodes must be labelled by numbers')
end

if sizeofA(2)~=2
    edgeList = edgeList';
end

match = edgeList(:,1) == edgeList(:,2);
edgeList(match,:) = []; % Removing self loops
toKeep = ismember(edgeList(:,2), edgeList(:,1)) & ismember(edgeList(:,1), edgeList(:,2));
x = edgeList(toKeep,:); % Removing edges that belong to nodes with zero indegree or zero outdegree
clearvars edgeList toKeep

minnode = min(x(:));
if minnode ~= 1
    x = x - (minnode-1); % Modify nodes labels so that the numbering starts from 1
end
nodes = (unique(x(:,1)))';

% Forming adjacency lists
maxnode = max(x(:));
adj = cell(1,maxnode); % adj(i) gives the list of nodes adjacent to node i.
dej = cell(1,maxnode); % dej(i) gives the list of nodes which have node i as adjacent node.
y = sortrows(x,1);
clearvars x
z = sortrows(y,2);
c = y(1,1);
cz = z(1,2);
beginrow = 1;
beginrowz = 1;
for i=1:size(y,1)
    if(y(i,1) ~= c)
        endrow = i-1;
        adj{c} = (y(beginrow:endrow,2))';
        beginrow = i;
        c = y(i,1);
    end
    if(z(i,2) ~= cz)
        endrowz = i-1;
        dej{cz} = (z(beginrowz:endrowz,1))';
        beginrowz = i;
        cz = z(i,2);
    end
end
yyy = y(:,1) == y(size(y,1),1);
zzz = z(:,2) == z(size(z,1),2);
adj{y(size(y,1),1)} = (y(yyy,2))';
dej{z(size(z,1),2)} = (z(zzz,1))';
clearvars y z

% Forming adj(adj(i))
lens = cellfun('length',adj);
arr = [adj{:}];
cumlens = cumsum(lens(arr));
grouped_cumlens = cumlens(cumsum(lens));
grouped_adj = mat2cell(cell2mat(adj(arr)),1,diff([0 grouped_cumlens]));
% grouped_adj(i) gives the list of nodes that are adjacent to nodes in adj(i) i.e. grouped_adj(i) is adj(adj(i))

% Counting Triangles
count = 0; % Initiating count
for i = nodes
    count = count + sum(ismember(grouped_adj{i},dej{i}));
end
number_of_triangles = count/3; 
% Each triangle is counted thrices (once for each node in it). So total number of triangles is count/3
end